@@ -667,11 +667,11 @@ impl FromStr for KeyCode {
   type Err = OsError;
   fn from_str(accelerator_string: &str) -> Result<Self, Self::Err> {
     let keycode = match accelerator_string.to_uppercase().as_str() {
-      "BACKQUOTE" => KeyCode::Backquote,
+      "`" | "BACKQUOTE" => KeyCode::Backquote,
       "BACKSLASH" => KeyCode::Backslash,
-      "BRACKETLEFT" => KeyCode::BracketLeft,
-      "BRACKETRIGHT" => KeyCode::BracketRight,
-      "COMMA" => KeyCode::Comma,
+      "[" | "BRACKETLEFT" => KeyCode::BracketLeft,
+      "]" | "BRACKETRIGHT" => KeyCode::BracketRight,
+      "," | "COMMA" => KeyCode::Comma,
       "0" => KeyCode::Digit0,
       "1" => KeyCode::Digit1,
       "2" => KeyCode::Digit2,
@@ -682,20 +682,20 @@ impl FromStr for KeyCode {
       "7" => KeyCode::Digit7,
       "8" => KeyCode::Digit8,
       "9" => KeyCode::Digit9,
-      "NUMPAD0" => KeyCode::Numpad0,
-      "NUMPAD1" => KeyCode::Numpad1,
-      "NUMPAD2" => KeyCode::Numpad2,
-      "NUMPAD3" => KeyCode::Numpad3,
-      "NUMPAD4" => KeyCode::Numpad4,
-      "NUMPAD5" => KeyCode::Numpad5,
-      "NUMPAD6" => KeyCode::Numpad6,
-      "NUMPAD7" => KeyCode::Numpad7,
-      "NUMPAD8" => KeyCode::Numpad8,
-      "NUMPAD9" => KeyCode::Numpad9,
+      "NUM0" | "NUMPAD0" => KeyCode::Numpad0,
+      "NUM1" | "NUMPAD1" => KeyCode::Numpad1,
+      "NUM2" | "NUMPAD2" => KeyCode::Numpad2,
+      "NUM3" | "NUMPAD3" => KeyCode::Numpad3,
+      "NUM4" | "NUMPAD4" => KeyCode::Numpad4,
+      "NUM5" | "NUMPAD5" => KeyCode::Numpad5,
+      "NUM6" | "NUMPAD6" => KeyCode::Numpad6,
+      "NUM7" | "NUMPAD7" => KeyCode::Numpad7,
+      "NUM8" | "NUMPAD8" => KeyCode::Numpad8,
+      "NUM9" | "NUMPAD9" => KeyCode::Numpad9,
       "=" => KeyCode::Equal,
       "-" => KeyCode::Minus,
-      "PERIOD" => KeyCode::Period,
-      "QUOTE" => KeyCode::Quote,
+      "." | "PERIOD" => KeyCode::Period,
+      "'" | "QUOTE" => KeyCode::Quote,
       "\\" => KeyCode::IntlBackslash,
       "A" => KeyCode::KeyA,
       "B" => KeyCode::KeyB,
@@ -724,8 +724,8 @@ impl FromStr for KeyCode {
       "Y" => KeyCode::KeyY,
       "Z" => KeyCode::KeyZ,
 
-      "SEMICOLON" => KeyCode::Semicolon,
-      "SLASH" => KeyCode::Slash,
+      ";" | "SEMICOLON" => KeyCode::Semicolon,
+      "/" | "SLASH" => KeyCode::Slash,
       "BACKSPACE" => KeyCode::Backspace,
       "CAPSLOCK" => KeyCode::CapsLock,
       "CONTEXTMENU" => KeyCode::ContextMenu,
@@ -747,16 +747,15 @@ impl FromStr for KeyCode {
       "RIGHT" => KeyCode::ArrowRight,
 
       "NUMLOCK" => KeyCode::NumLock,
-      "NUMPADADD" => KeyCode::NumpadAdd,
-      "NUMPADBACKSPACE" => KeyCode::NumpadBackspace,
-      "NUMPADCLEAR" => KeyCode::NumpadClear,
-      "NUMPADCOMMA" => KeyCode::NumpadComma,
-      "NUMPADDIVIDE" => KeyCode::NumpadDivide,
-      "NUMPADSUBSTRACT" => KeyCode::NumpadSubtract,
-      "NUMPADENTER" => KeyCode::NumpadEnter,
+      "NUMADD" | "NUMPADADD" => KeyCode::NumpadAdd,
+      "NUMBACKSPACE" | "NUMPADBACKSPACE" => KeyCode::NumpadBackspace,
+      "NUMCLEAR" | "NUMPADCLEAR" => KeyCode::NumpadClear,
+      "NUMCOMMA" | "NUMPADCOMMA" => KeyCode::NumpadComma,
+      "NUMDIVIDE" | "NUMPADDIVIDE" => KeyCode::NumpadDivide,
+      "NUMSUBSTRACT" | "NUMPADSUBSTRACT" => KeyCode::NumpadSubtract,
+      "NUMENTER" | "NUMPADENTER" => KeyCode::NumpadEnter,
 
-      "ESCAPE" => KeyCode::Escape,
-      "ESC" => KeyCode::Escape,
+      "ESC" | "ESCAPE" => KeyCode::Escape,
       "FN" => KeyCode::Fn,
       "FNLOCK" => KeyCode::FnLock,
       "PRINTSCREEN" => KeyCode::PrintScreen,
